<?php
/**
 * 2007-2021 PrestaShop
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 * @author    PrestaShop SA <contact@prestashop.com>
 * @copyright 2007-2021 PrestaShop SA
 * @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 *  International Registered Trademark & Property of PrestaShop SA
 */

/**
 * Class KoalapayCustomerDebtor
 */
class KoalapayCustomerDebtor extends ObjectModel
{
    /** @var int $id Customer Debtor ID */
    public $id;

    /** @var int $id_customer Customer ID */
    public $id_customer;

    /** @var string $id_debtor debtor ID */
    public $id_debtor;

    /**
     * @see ObjectModel::$definition
     */
    public static $definition = array(
        'table' => 'koalapay_customer_debtor',
        'primary' => 'id_customer_debtor',
        'fields' => array(
            'id_customer' => array('type' => self::TYPE_INT, 'validate' => 'isUnsignedInt', 'required' => true),
            'id_debtor' => array(
                'type' => self::TYPE_STRING,
                'validate' => 'isString',
                'required' => true,
                'size' => 255
            ),
        ),
    );

    /**
     * Get Debtor ID by Customer
     * @param mixed $customer Customer or ID
     * @return string debtor ID
     */
    public static function getIdDebtor($customer)
    {
        if (Validate::isLoadedObject($customer)) {
            $id_customer = $customer->id;
        } else {
            $id_customer = $customer;
        }
        $sql = 'SELECT `id_debtor`
			FROM `' . _DB_PREFIX_ . 'koalapay_customer_debtor`
			WHERE `id_customer` = ' . (int)$id_customer;
        return Db::getInstance()->getValue($sql);
    }

    /**
     * Get Customer ID by debtor
     * @param string $id_debtor Debtor ID
     * @return int customer ID
     */
    public static function getIdCustomer($id_debtor)
    {
        $sql = 'SELECT `id_customer`
			FROM `' . _DB_PREFIX_ . 'koalapay_customer_debtor`
			WHERE `id_debtor` = ' . (int)$id_debtor;
        return Db::getInstance()->getValue($sql);
    }

    /**
     * Delete Customer Debtor
     * @param mixed $customer Customer or ID customer
     * @return bool
     */
    public static function deleteByCustomer($customer)
    {
        if (Validate::isLoadedObject($customer)) {
            $id_customer = $customer->id;
        } else {
            $id_customer = $customer;
        }
        return Db::getInstance()->delete('koalapay_customer_debtor', 'id_customer = ' . (int)$id_customer);
    }
}
