<?php
/**
 * 2007-2021 PrestaShop
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 * @author    PrestaShop SA <contact@prestashop.com>
 * @copyright 2007-2021 PrestaShop SA
 * @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 *  International Registered Trademark & Property of PrestaShop SA
 */

/*
    This comment is useful to the translator tools /!\ do not remove them

    $this->l('draft');
    $this->l('sent');
    $this->l('paid');
    $this->l('reversed');
    $this->l('disputed');
    $this->l('reminder1');
    $this->l('reminder2');
    $this->l('reminder3');
    $this->l('waitingForDebtor');
    $this->l('waitingForApproval');
    $this->l('financed');
    $this->l('directPayment');
    $this->l('canceled');
    $this->l('expired');
    $this->l('closed');
    $this->l('refused');
*/

/**
 * Class AdminKoalapayOrdersController
 */
class AdminKoalapayOrdersController extends ModuleAdminController
{
    public $lang = false;
    public $table = 'order';
    public $className = 'Order';
    public $bootstrap = true;
    public $allow_export = true;
    public $actions_available = array('capture', 'view');
    public $capture = 1;
    public $edit = 1;
    public $view = 0;
    public $delete = 0;
    public $bulk_actions = array();

    /**
     * @var Koalapay
     */
    public $module;

    protected $return_code_list = null;
    protected $order_statuses = array();
    protected $invoice_statuses = array();

    public function __construct()
    {
        parent::__construct();

        $this->_conf[101] = $this->l('Invoice(s) were sent succesfully');
        $this->_conf[102] = $this->l('Status of Invoices for selected orders were refreshed succesfully');

        foreach (OrderState::getOrderStates((int)$this->context->language->id) as $status) {
            $this->order_statuses[$status['id_order_state']] = $status['name'];
        }
        $this->actions = array('view');

        $statuses = KoalapayInvoiceStatus::getStatuses();
        foreach ($statuses as $status) {
            $this->invoice_statuses[$status['name']] = $this->l($status['name']);
        }
        $statuses = KoalapayInvoiceFinancingStatus::getStatuses();
        foreach ($statuses as $status) {
            $this->invoice_fiancing_statuses[$status['name']] = $this->l($status['name']);
        }

        $this->setFieldsListDefinition();
        $this->setQuery();

        $api_alive = Kpay::apiAlive();
        if (Configuration::get('KOALAPAY_API_TOKEN') && $api_alive === true) {
            $this->addRowAction('send');
            $this->addRowAction('refresh');
            $this->bulk_actions['send'] = array(
                'text' => $this->l('Send Invoice'),
                'icon' => 'icon-envelope',
                'confirm' => $this->l('Do you want send invoice for selected orders?'),
            );
            $this->bulk_actions['refresh'] = array(
                'text' => $this->l('Refresh Invoices Status'),
                'icon' => 'icon-refresh',
                'confirm' => $this->l('Do you want refresh status invoice for selected orders?'),
            );
        } else {
            if ($api_alive) {
                $this->warnings[] = $this->l('Your account is not configured.');
            } else {
                $this->warnings[] = $this->l('API is down!? Come back later.');
            }
        }
    }

    public function initToolbar()
    {
        parent::initToolbar();
        unset($this->toolbar_btn['new']);
    }

    protected function setFieldsListDefinition()
    {
        $this->fields_list['id_order'] = array(
            'class' => 'fixed-width-s',
            'type' => 'text',
            'title' => $this->l('Id order'),
            'name' => 'id_order',
            'search' => true,
            'orderby' => true,
        );
        $this->fields_list['reference'] = array(
            'class' => 'fixed-width-sm',
            'title' => $this->l('Order reference'),
            'type' => 'text',
            'search' => true,
            'orderby' => true,
        );
        $this->fields_list['osname'] = array(
            'title' => $this->l('Status'),
            'type' => 'select',
            'color' => 'color',
            'list' => $this->order_statuses,
            'filter_key' => 'os!id_order_state',
            'filter_type' => 'int',
            'order_key' => 'osname',
            'search' => true,
            'orderby' => true,
        );
        $this->fields_list['date_add'] = array(
            'class' => 'fixed-width-s',
            'title' => $this->l('Date'),
            'type' => 'date',
            'search' => true,
            'orderby' => true,
        );
        $this->fields_list['id_cart'] = array(
            'class' => 'fixed-width-s',
            'type' => 'text',
            'title' => $this->l('Id Cart'),
            'name' => 'id_cart',
            'filter_key' => 'a!id_cart',
            'search' => true,
            'orderby' => true,
        );
        $this->fields_list['invoice_number'] = array(
            'class' => 'fixed-width-s',
            'type' => 'text',
            'title' => $this->l('Invoice #'),
            'name' => 'invoice_number',
            'filter_key' => 'ki!invoice_number',
            'search' => true,
            'orderby' => false,
        );
        $this->fields_list['id_pdf'] = array(
            'title' => $this->l('PDF'),
            'align' => 'text-center',
            'callback' => 'printPDFIcons',
            'orderby' => false,
            'search' => false,
            'remove_onclick' => true,
        );
        $this->fields_list['total'] = array(
            'title' => $this->l('Total'),
            'align' => 'text-right',
            'type' => 'price',
            'currency' => true,
            'callback' => 'setOrderCurrency',
            'badge_success' => true,
        );
        $this->fields_list['status'] = array(
            'title' => $this->l('Invoice status'),
            'type' => 'select',
            'color' => 'status_color',
            'callback' => 'setTranslation',
            'list' => $this->invoice_statuses,
            'filter_key' => 'ki!status',
            'order_key' => 'kis!id_invoice_status',
            'search' => true,
            'orderby' => true,
        );
        $this->fields_list['financing_status'] = array(
            'title' => $this->l('Financing status'),
            'type' => 'select',
            'color' => 'financing_status_color',
            'callback' => 'setTranslation',
            'list' => $this->invoice_fiancing_statuses,
            'filter_key' => 'ki!financing_status',
            'order_key' => 'kifs!id_invoice_financing_status',
            'search' => true,
            'orderby' => true,
        );
    }

    protected function setQuery()
    {
        $this->_select .= 'a.id_order AS id_pdf, os.color, osl.name AS osname, ';
        $this->_select .= 'ki.id_invoice, ki.invoice_number, ki.status, ki.financing_status, ki.total, ';
        $this->_select .= 'kis.id_invoice_status, kis.color as status_color, ';
        $this->_select .= 'kifs.id_invoice_financing_status, kifs.color as financing_status_color';

        $this->_join .= ' 
        INNER JOIN `' . _DB_PREFIX_ . 'koalapay_cart_invoice` ki ON ki.`id_cart` = a.`id_cart`
        LEFT JOIN `' . _DB_PREFIX_ . 'koalapay_invoice_status` kis ON kis.`name` = ki.`status`
        LEFT JOIN `' . _DB_PREFIX_ . 'koalapay_invoice_financing_status` kifs ON kifs.`name` = ki.`financing_status`
        LEFT JOIN `' . _DB_PREFIX_ . 'order_state` os ON (os.`id_order_state` = a.`current_state`)
        LEFT JOIN `' . _DB_PREFIX_ . 'order_state_lang` osl ON ( 
            os.`id_order_state` = osl.`id_order_state` AND osl.`id_lang` = ' . (int)$this->context->language->id.'
        )';

        $this->_where .= ' AND a.module = "' . pSQL($this->module->name) . '"';

        $this->_orderBy = 'id_order';
        $this->_orderWay = 'DESC';
    }

    public function renderView()
    {
        $id_order = Tools::getValue('id_order');
        if ($id_order) {
            $token = Tools::getAdminTokenLite('AdminOrders');
            Tools::redirectAdmin('index.php?tab=AdminOrders&id_order=' . $id_order . '&vieworder&token=' . $token);
        } else {
            $this->renderList();
        }
    }

    public function processSend()
    {
        $id_order = Tools::getValue('id_order');
        if ($this->tabAccess['edit'] != 1 && !$this->context->employee->isSuperAdmin()) {
            $this->errors[] = Tools::displayError('You do not have permission to edit this.');
            return false;
        }

        if (!$id_order) {
            $this->errors[] = Tools::displayError('Invalid Order Id.');
            return false;
        }

        $this->sendInvoice($id_order);

        return true;
    }

    public function processBulkSend()
    {
        if ($this->tabAccess['edit'] != 1 && !$this->context->employee->isSuperAdmin()) {
            $this->errors[] = Tools::displayError('You do not have permission to edit this.');
            return false;
        }

        if (!is_array($this->boxes) || empty($this->boxes)) {
            return false;
        }

        foreach ($this->boxes as $id_order) {
            $this->sendInvoice($id_order);
        }

        if (!empty($this->confirmations) && empty($this->errors)) {
            Tools::redirectAdmin(AdminController::$currentIndex . '&conf=101&token=' . $this->token);
        }

        return true;
    }

    public function processRefresh()
    {
        $id_order = Tools::getValue('id_order');
        if ($this->tabAccess['edit'] != 1 && !$this->context->employee->isSuperAdmin()) {
            $this->errors[] = Tools::displayError('You do not have permission to edit this.');
            return false;
        }

        if (!$id_order) {
            $this->errors[] = Tools::displayError('Invalid Order Id.');
            return false;
        }

        $this->refreshStatus($id_order);

        return true;
    }

    public function processBulkRefresh()
    {
        if ($this->tabAccess['edit'] != 1 && !$this->context->employee->isSuperAdmin()) {
            $this->errors[] = Tools::displayError('You do not have permission to edit this.');
            return false;
        }

        if (!is_array($this->boxes) || empty($this->boxes)) {
            return false;
        }

        foreach ($this->boxes as $id_order) {
            $this->refreshStatus($id_order);
        }

        if (!empty($this->confirmations) && empty($this->errors)) {
            Tools::redirectAdmin(AdminController::$currentIndex . '&conf=102&token=' . $this->token);
        }

        return true;
    }

    protected function sendInvoice($id_order)
    {
        try {
            $id_order = (int)$id_order;
            $order = new Order($id_order);
            if (!Validate::isLoadedObject($order)) {
                throw new Exception(sprintf(
                    $this->l('Order not found (ID %s)'),
                    $id_order
                ));
            }

            $cart = new Cart($order->id_cart);
            if (!Validate::isLoadedObject($cart)) {
                throw new Exception(sprintf(
                    $this->l('Cart not found (ID %s) for Order (ID %s)'),
                    $order->id_cart,
                    $id_order
                ));
            }

            Kpay::sendInvoice($cart);

            $os = new OrderHistory();
            $os->id_order = $id_order;
            $os->changeIdOrderState(Configuration::get('KOALAPAY_OS_PAYMENT'), $id_order);
            $os->save();

            $message = sprintf($this->l('Invoice sent for Order (ID %s)'), $id_order);
            $this->confirmations[] = $message;
            $this->module->log($message, 1, null, 'Order', $id_order);
        } catch (Exception $e) {
            $message = sprintf($this->l('Unable to send invoice: %s'), $e->getMessage());
            $this->errors[] = $message;
            $this->module->log($message, 3, null, 'Order', $id_order);
        }
    }

    protected function refreshStatus($id_order)
    {
        try {
            $order = new Order($id_order);
            if (!Validate::isLoadedObject($order)) {
                throw new Exception(sprintf(
                    $this->l('Order not found (ID %s)'),
                    $id_order
                ));
            }

            $cart = new Cart($order->id_cart);
            if (!Validate::isLoadedObject($cart)) {
                throw new Exception(sprintf(
                    $this->l('Cart not found (ID %s) for Order (ID %s)'),
                    $order->id_cart,
                    $id_order
                ));
            }

            Kpay::refreshStatus($cart);

            $message = sprintf($this->l('Status of invoice were refreshed for Cart (ID %s)'), $cart->id);
            $this->confirmations[] = $message;
            $this->module->log($message, 1, null, 'Order', $id_order);
        } catch (Exception $e) {
            $message = sprintf($this->l('Unable to refresh status of invoice: %s'), $e->getMessage());
            $this->errors[] = $message;
            $this->module->log($message, 3, null, 'Order', $id_order);
        }
    }

    public function setOrderCurrency($echo, $tr)
    {
        if (!empty($tr['id_currency'])) {
            $idCurrency = (int)$tr['id_currency'];
        } else {
            $order = new Order($tr['id_order']);
            $idCurrency = (int)$order->id_currency;
        }

        return Tools::displayPrice($echo, $idCurrency);
    }

    public function printPDFIcons($id_order, $tr)
    {
        $order = new Order($id_order);
        if (!Validate::isLoadedObject($order)) {
            return '';
        }

        $this->context->smarty->assign(array(
            'order' => $order,
            'tr' => $tr,
            'show_invoice_pdf' => !in_array($tr['status'], array('draft', 'canceled', ''))
        ));

        $tpl = 'helpers/list/print_pdf_icon.tpl';

        if (!$this->module->isPS17x()) {
            // backward compatibility
            $tpl = _PS_MODULE_DIR_ . $this->module->name.'/views/templates/admin/' . $tpl;
            return $this->context->smarty->createTemplate($tpl, $this->context->smarty)->fetch();
        }

        return $this->createTemplate($tpl)->fetch();
    }

    public function setTranslation($echo)
    {
        return $this->l($echo);
    }
}
