<?php
/**
 * 2007-2021 PrestaShop
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 * @author    PrestaShop SA <contact@prestashop.com>
 * @copyright 2007-2021 PrestaShop SA
 * @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 *  International Registered Trademark & Property of PrestaShop SA
 */

/**
 * Class KoalapayValidationModuleFrontController
 */
class KoalapayValidationModuleFrontController extends ModuleFrontController
{

    public function postProcess()
    {
        $cart = $this->context->cart;
        $id_module = $this->module->id;
        $module_name = $this->module->name;

        // Check that this payment option is still available in case the customer changed his address
        // just before the end of the checkout process
        $authorized = false;
        foreach (Module::getPaymentModules() as $module) {
            if ($module['name'] == $module_name) {
                $authorized = true;
                break;
            }
        }

        if (!$authorized) {
            die($this->module->l('This payment method is not available.'));
        }

        $result = $this->module->preparePayment($cart);
        if (!$result['show_method']) {
            Tools::redirect('index.php?controller=order&step=1');
        }
        $invoice = $result['invoice'];
        $debtor = $result['debtor'];

        $order_total = $cart->getOrderTotal();

        $payment_status = Configuration::get('PS_OS_ERROR');
        $mail_vars = array(
            '{koalapay_invoice_id}' => $invoice->id,
            '{koalapay_debtor_id}' => $debtor->id,
            '{koalapay_debtor_name}' => implode(', ', $debtor->names)
        );
        if (Configuration::get('KOALAPAY_SEND_INVOICE_IMMEDIATELY')) {
            try {
                $invoice_updated = Kpay::sendInvoice($cart);
                $payment_status = Configuration::get('KOALAPAY_OS_PAYMENT');
                $message = sprintf($this->module->l('Invoice %s sent to debtor.'), $invoice_updated->id);
                $mail_vars['{koalapay_invoice_id}'] = $invoice_updated->id;
                $order_total = $invoice_updated->totals->total;
            } catch (Exception $e) {
                $log = sprintf(
                    $this->module->l('Unable to send invoice %s to debtor %s (ID %s)'),
                    $mail_vars['{koalapay_invoice_id}'],
                    $mail_vars['{koalapay_debtor_name}'],
                    $mail_vars['{koalapay_debtor_id}']
                );
                $log .= sprintf($this->module->l(', %s'), $e->getMessage());
                $message = $log;
                $this->module->log($log, 2, null, 'Cart', $cart->id);
            }
        } else {
            $payment_status = Configuration::get('KOALAPAY_OS_PAYMENT_PENDING');
            $message = sprintf(
                $this->module->l('Invoice %s was not sent to debtor %s (ID %s).'),
                $mail_vars['{koalapay_invoice_id}'],
                $mail_vars['{koalapay_debtor_name}'],
                $mail_vars['{koalapay_debtor_id}']
            );
        }

        $this->module->validateOrder(
            $cart->id,
            $payment_status,
            $order_total,
            $this->module->displayName,
            $message,
            $mail_vars,
            null,
            false,
            $cart->secure_key
        );

        $url = 'index.php?controller=order-confirmation';
        $url .= '&id_cart=' . $cart->id;
        $url .= '&id_module=' . (int)$id_module;
        $url .= '&id_order=' . $this->module->currentOrder;
        $url .= '&key=' . $cart->secure_key;
        Tools::redirect($url);
    }
}
